import uuid
from django.contrib.auth.models import AbstractUser
from django.db.models import *
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _
from djrichtextfield.models import RichTextField
from fcm_django.models import FCMDevice
from django.core.validators import FileExtensionValidator
from django.core.exceptions import ValidationError


def validate_file_size(value):
    filesize= value.size    
    if filesize > 104857600:
        raise ValidationError("You cannot upload file more than 100Mb")
    else:
        return value
    
class User(AbstractUser):
    id = UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    username = CharField(blank=True, null=True, max_length=20)
    firstname = CharField(blank=True, null=True, max_length=20)
    lastname = CharField(blank=True, null=True, max_length=20)
    phone = CharField(_("phone"), max_length=64, blank=True, null=True)
    work = TextField(default="", blank=True, null=True)
    gender = CharField(
        default="", blank=True, max_length=20, null=True, verbose_name="gender"
    )
    email = EmailField(_("email address"), unique=True)
    image = ImageField(upload_to="user_img", null=True, blank=True)
    jwt_secret = UUIDField(default=uuid.uuid4, editable=False)
    isDeleted = BooleanField(default=False)
    listing_status = BooleanField(default=True)
    cancel_subscription_applied = BooleanField(default=False)
    created_at = DateField(auto_now_add=True)
    updated_at = DateField(auto_now=True)
    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = ["username"]

    def __str__(self):
        return "{}".format(self.email)

    @property
    def check_trail_subscription(self):
        import datetime

        if datetime.date.today() > self.created_at + datetime.timedelta(days=7):
            check_trail = UserSubscription.objects.filter(user=self)
            if check_trail.exists():
                return check_trail.first().check_subcription()
            return False
        else:
            return True

    @property
    def download_songs_count(self):
        return DownloadedSongs.objects.filter(user=self).aggregate(Count('songs'))['songs__count']


class UserNotification(Model):
    user = ForeignKey(
        User,
        related_name="user_notification_list",
        on_delete=CASCADE,
        null=True,
        blank=True,
    )
    image = ImageField(upload_to="user_notification_images")
    title = TextField(default="", blank=True, null=True, verbose_name="title")
    message = TextField(default="", blank=True,
                        null=True, verbose_name="message")
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "user_notification"

    def __str__(self):
        return "{}".format(self.title)


class OTP(Model):
    user = ForeignKey(
        User, related_name="user_otp", on_delete=CASCADE, null=True, blank=True
    )
    otp = TextField(default="", blank=False, null=False)
    ip = TextField(default="", blank=False, null=False)
    status = TextField(
        default="",
        blank=True,
        null=True,
    )
    isVerified = BooleanField(blank=False, default=False)
    counter = IntegerField(default=0, blank=False)
    isDeleted = BooleanField(default=False)

    class Meta:
        db_table = "user_otp"


class Recommendation(Model):
    name = TextField(
        default="",
        blank=True,
        null=True,
    )

    class Meta:
        db_table = "recommendation"

    def __str__(self):
        return "{}".format(self.name)


class Personlize(Model):
    user = ForeignKey(
        User, related_name="user_personlize", on_delete=CASCADE,
    )
    recommendation = ManyToManyField(Recommendation, blank=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "personlize"

    def __str__(self):
        return "{}".format(self.id)


class CategoryImage(Model):
    image = ImageField(upload_to="category_images")

    def get_image_url(self):
        from django.utils.html import format_html
        return format_html(
            f'<img src="{self.image.url}" width="100" height="100"/>'
        )

    class Meta:
        db_table = "category_image"

    def __str__(self):
        return "{}".format(self.id)


class SongsCategory(Model):
    name = CharField(max_length=200)
    title = CharField(max_length=200)
    # images = ManyToManyField(CategoryImage, blank=True)
    cover_images = ImageField(upload_to="category_images")
    description = TextField(default="", blank=True, null=True)
    author_name = CharField(max_length=200)
    author_image = ImageField(upload_to="category_images")
    # listen_count = IntegerField(default="0", blank=True, null=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "songs_category"
        verbose_name_plural = "Song Category"

    def __str__(self):
        return "{}".format(self.name)



class SongsImage(Model):
    image = ImageField(upload_to="songs_images")

    class Meta:
        db_table = "songs_image"

    def __str__(self):
        return "{}".format(self.id)

    def get_image_url(self):
        from django.utils.html import format_html
        return format_html(
            f'<img src="{self.image.url}" width="100" height="100"/>'
        )


class Songs(Model):
    songs_category = ForeignKey(
        SongsCategory,
        related_name="category_songs",
        on_delete=CASCADE
    )
    name = CharField(max_length=200)
    title = CharField(max_length=200)
    # images = ManyToManyField(SongsImage, blank=True)
    cover_images = ImageField(upload_to="songs_images")
    audios = FileField(upload_to="audios", validators=[FileExtensionValidator(allowed_extensions=[
                       "mp3", 'wav', 'wave', 'mpga', 'midi', 'aa3', 'mpu', 'hca', 'aac', 'm4p']), validate_file_size])
    description = TextField(default="", blank=True, null=True)
    listen_count = IntegerField(default="0", blank=True, null=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "songs"
        verbose_name_plural = "songs"

    def __str__(self):
        return "{}".format(self.name)


@receiver(post_save, sender=Songs)
def notification(sender, created, instance, **kwargs):
    if created:
        users = User.objects.all()
        notification_devices = FCMDevice.objects.filter(
            user__in=users, active=True)
        users_list = notification_devices.values_list(
            "user__id", flat=True).distinct()
        if notification_devices:
            ios_devices = notification_devices.filter(type="ios")
            android_devices = notification_devices.filter(type="android")
            data = {"title": "New Song is added", "body": "New Song is added"}
            kwargs = {
                "content_available": True,
                "extra_kwargs": {
                    "priority": "high",
                    "mutable_content": True,
                    "notification": data,
                },
            }
            ios_stats = ios_devices.send_message(sound="default", **kwargs)
            android_stats = android_devices.send_message(
                data={
                    "title": "New Song is added",
                    "body": "New Song is added",
                    "type": "Match",
                }
            )
            try:
                ios_success = ios_stats.get("success")
            except:
                ios_success = 0
            try:
                android_success = android_stats.get("success")
            except:
                android_success = 0
            print("ios notification stats", ios_success, ios_stats)
            print("android notification stats", android_success, android_stats)
            if ios_success > 0 or android_success > 0:
                try:
                    for user_id in users_list:
                        try:
                            user = User.objects.get(id=user_id)
                        except:
                            user = None
                        if user:
                            UserNotification.objects.create(
                                user=user,
                                title="New Song is added",
                                message="New Song is added",
                                image=instance.cover_images,
                            )
                except:
                    pass
    pass


class TodaySpecialFile(Model):
    file = FileField(upload_to="today_special", verbose_name="Video", validators=[FileExtensionValidator(allowed_extensions=[
                       'mp4', 'wma', 'm4u', 'mpeg-2', 'mkv', 'mov', 'webm', 'avi', 'pgi', 'mpv']), validate_file_size])
    listen_count = IntegerField(default="0", blank=True, null=True)
    name = CharField(max_length=200)
    title = CharField(max_length=200)
    # status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "today_special_file"

    def __str__(self):
        return "{}".format(self.name)


class TodaySpecial(Model):
    video = ForeignKey(TodaySpecialFile, on_delete=SET_NULL,
                       blank=True, null=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "today_special"

    def __str__(self):
        return "{}".format(self.id)
    
    def save(self, *args, **kwargs):
        self.__class__.objects.all().update(status=False)
        self.status = True
        super().save(*args, **kwargs)


class UserFavourite(Model):
    user = ForeignKey(
        User, related_name="user_favourite", on_delete=CASCADE,
    )
    songs = ForeignKey(
        Songs,
        related_name="user_favourite_songs",
        on_delete=CASCADE,
    )
    listen_count = IntegerField(default="0", blank=True, null=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "user_favourite"


class UserVideoFavourite(Model):
    user = ForeignKey(
        User, on_delete=CASCADE,
    )
    videos = ForeignKey(
        'ExploreVideos',
        related_name="user_favourite_songs",
        on_delete=CASCADE
    )
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "user_favourite_video"


class EffectCategory(Model):
    name = CharField(max_length=200)
    title = CharField(max_length=200)
    cover_images = ImageField(upload_to="effect_category_images")
    description = TextField(default="", blank=True, null=True)
    author_name = CharField(max_length=200)
    author_image = ImageField(upload_to="effect_category_images")
    # listen_count = IntegerField(default="0", blank=True, null=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "effect_category"
        verbose_name_plural = "Effect Category"

    def __str__(self):
        return "{}".format(self.name)


class EffectAlbum(Model):
    effect_category = ForeignKey(
        EffectCategory,
        related_name="category_effects_albums",
        on_delete=CASCADE
    )
    name = CharField(max_length=200)
    title = CharField(max_length=200)
    cover_images = ImageField(upload_to="effect_album_images")
    description = TextField(default="", blank=True, null=True)
    # listen_count = IntegerField(default="0", blank=True, null=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "effect_album"

    def __str__(self):
        return "{}".format(self.name)


class EffectSongs(Model):
    effect_album = ForeignKey(
        EffectAlbum,
        related_name="effect_albums_songs",
        on_delete=CASCADE
    )
    name = CharField(max_length=200)
    title = CharField(max_length=200)
    cover_images = ImageField(upload_to="effect_songs_images")
    audios = FileField(upload_to="effect_audios", validators=[FileExtensionValidator(allowed_extensions=[
                       "mp3", 'wav', 'wave', 'mpga', 'midi', 'aa3', 'mpu', 'hca', 'aac', 'm4p']),validate_file_size])
    description = TextField(default="", blank=True, null=True)
    # listen_count = IntegerField(default=0, null=True, blank=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "effect_songs"
        verbose_name_plural = "effect songs"

    def __str__(self):
        return "{}".format(self.name)


class ExploreVideos(Model):
    name = CharField(max_length=200)
    file = FileField(upload_to="explore_videos",validators=[FileExtensionValidator(allowed_extensions=[
                       'mp4', 'wma', 'm4u', 'mpeg-2', 'mkv', 'mov', 'webm', 'avi', 'pgi', 'mpv']),validate_file_size])

    class Meta:
        db_table = "explore_videos"
        verbose_name_plural = "explore videos"

    def __str__(self):
        return "{}".format(self.name)


class ExploreSongs(Model):
    songs = ManyToManyField(Songs, related_name="songs_explore", blank=True)
    videos = ManyToManyField(
        ExploreVideos, related_name="videos_explore", blank=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "explore_songs"
        verbose_name_plural = "explore songs"

    def __str__(self):
        return "{}".format(self.id)


class FavouriteVideos(Model):
    user = ForeignKey(
        User,
        related_name="favourite_video_list",
        on_delete=CASCADE,
        null=True,
        blank=True,
    )
    videos = ForeignKey(
        ExploreVideos,
        related_name="favourite_video",
        on_delete=CASCADE,
        null=True,
        blank=True,
    )
    listen_count = IntegerField(default="0", blank=True, null=True)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "favourite_videos"


class Page(Model):
    title = CharField(max_length=250, verbose_name="title")
    pagerole = CharField(max_length=250, verbose_name="pagerole")
    description = RichTextField(
        default="", blank=True, null=True, verbose_name="description"
    )
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "page"
        verbose_name_plural = "pages"

    def __str__(self):
        return "{}".format(self.title)


class Contact(Model):
    first_name = CharField(max_length=200, verbose_name="first_name")
    last_name = CharField(max_length=200, verbose_name="last_name")
    email = EmailField(default="", verbose_name="email")
    phone = CharField(default="", max_length=200, verbose_name="phone")
    message = TextField(default="", blank=True,
                        null=True, verbose_name="message")
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "contact"

    def __str__(self):
        return "{}".format(self.email)


class LoginPageVideo(Model):
    video = FileField(upload_to="login_page_video", validators=[FileExtensionValidator(
        allowed_extensions=['mp4', 'wma', 'm4u', 'mpeg-2', 'mkv', 'mov', 'webm', 'avi', 'pgi', 'mpv']), validate_file_size])
    name = CharField(max_length=200)
    status = BooleanField(default=True)
    isDeleted = BooleanField(default=False)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "login_page_video"

    def __str__(self):
        return "{}".format(self.name)
    
    def save(self, *args, **kwargs):
        self.__class__.objects.all().update(status=False)
        self.status = True
        super().save(*args, **kwargs)


class UserSubscription(Model):
    user = OneToOneField(
        User, related_name="subscription_user", on_delete=CASCADE)
    receipt = CharField(max_length=255, verbose_name="Receipt")
    productid = CharField(max_length=50, verbose_name="product id")
    packageName = CharField(max_length=50)
    status = BooleanField(default=True)
    platform = CharField(max_length=50, blank=True)
    transactionid = CharField(max_length=100, blank=True)
    purchaseDate = DateField(null=True, blank=True)
    expirationDate = DateField(null=True, blank=True)
    autorenewing = BooleanField(default=False)
    price = CharField(max_length=50, blank=True, null=True)
    currency_code = CharField(max_length=20, blank=True, null=True)
    country_code = CharField(max_length=20, blank=True, null=True)
    meta = TextField(null=True, blank=True)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    class Meta:
        db_table = "user_subcription"
        ordering = ["-created"]

    def __str__(self):
        return "{}".format(self.id)

    def check_subcription(self):
        from MeditationApp.utils import validate_subcription

        subscription = validate_subcription(
            {
                "platform": self.platform,
                "receipt": self.receipt,
                "productid": self.productid,
                "packageName": self.packageName,
            }
        )
        UserSubscription.objects.filter(user=self.user).update(
            status=subscription["success"]
        )
        return subscription["success"]

    def cancelsubcription(self):
        from MeditationApp.utils import cancel_subcription

        subscription = cancel_subcription(
            {
                "platform": self.platform,
                "receipt": self.receipt,
                "productid": self.productid,
                "packageName": self.packageName,
            }
        )
        return subscription["success"]


class DownloadedSongs(Model):
    DEVICE_TYPES = (
        (u'ios', u'ios'),
        (u'android', u'android'),
        (u'web', u'web')
    )

    user = ForeignKey(User, on_delete=CASCADE)
    device_id = CharField(
        verbose_name=_("Device ID"),
        help_text=_("Unique device identifier"),
        max_length=150
    )
    device_type = CharField(choices=DEVICE_TYPES, max_length=10)
    songs = ManyToManyField(Songs, blank=True)
    created = DateTimeField(auto_now_add=True)
    updated = DateTimeField(auto_now=True)

    def __str__(self): return str(self.id)

    class Meta:
        db_table = "downloaded_songs"
        verbose_name = "Downloaded Song"
        verbose_name_plural = "Downloaded Songs"
        ordering = ["-created"]
